@@ -3,6 +3,7 @@ import collections
 import hashlib
 import logging
 import progressbar
+import re
 import requests
 import socket
 import sqlite3
@@ -59,6 +60,18 @@ def ensure_local(*, base_url, entry, local_path):
         raise ValueError(f"Hash mismatch on downloaded file {local_path}")
 
 
+comment_re = re.compile(r"^\s*[#;]")
+
+
+def parse_hosts_file(fd):
+    host_strings = []
+    for l in fd:
+        line = l.strip()
+        if line and not comment_re.match(line):
+            host_strings.append(line)
+    return host_strings
+
+
 class IntermediatesDB(object):
     def __init__(self, *, db_path, download_pems=False):
         self.db_path = Path(db_path).expanduser()
@@ -388,10 +401,22 @@ class CRLiteQueryResult(object):
         self.via_stash = None
         self.via_filter = None
 
+        if not issuer:
+            self.state = "Unknown Issuer"
+            return
+
         if not issuer["crlite_enrolled"]:
             self.state = "Not Enrolled"
             return
 
+        if crlite_db.latest_covered_date() < not_before:
+            self.state = "Too New"
+            return
+
+        if crlite_db.latest_covered_date() > not_after:
+            self.state = "Expired"
+            return
+
         rev_status = crlite_db.revocation_status(self.cert_id)
 
         if rev_status["revoked"] is True:
@@ -402,14 +427,6 @@ class CRLiteQueryResult(object):
             self.state = "Revoked"
             return
 
-        if crlite_db.latest_covered_date() < not_before:
-            self.state = "Too New"
-            return
-
-        if crlite_db.latest_covered_date() > not_after:
-            self.state = "Expired"
-            return
-
         self.state = "Valid"
 
     def __str__(self):
@@ -422,33 +439,59 @@ class CRLiteQueryResult(object):
         if self.state == "Revoked":
             return "⛔️"
         if self.state == "Not Enrolled":
-            return "❌"
+            return "❔"
         if self.state == "Valid":
             return "👍"
         if self.state == "Too New":
             return "🐇"
         if self.state == "Expired":
             return "⏰"
+        if self.state == "Unknown Issuer":
+            return "⁉️"
         return ""
 
     def print_query_result(self):
         padded_name = self.name + " " * 5
         padding = "".ljust(len(padded_name))
 
-        enrolled_icon = "✅" if self.issuer["crlite_enrolled"] else "❌"
+        if not self.issuer:
+            self.state = "Unknown Issuer"
+        else:
+            enrolled_icon = "✅" if self.issuer["crlite_enrolled"] else "❌"
 
-        print(f"{padded_name} Issuer: {self.issuer['subject']}")
-        print(f"{padding} Enrolled in CRLite: {enrolled_icon}")
-        print(f"{padding} {self.cert_id}")
-        if self.via_filter:
-            print(f"{padding} Revoked via CRLite filter: {self.via_filter}")
-        if self.via_stash:
-            print(f"{padding} Revoked via Stash: {self.via_stash}")
+            print(f"{padded_name} Issuer: {self.issuer['subject']}")
+            print(f"{padding} Enrolled in CRLite: {enrolled_icon}")
+            print(f"{padding} {self.cert_id}")
+            if self.via_filter:
+                print(f"{padding} Revoked via CRLite filter: {self.via_filter}")
+            if self.via_stash:
+                print(f"{padding} Revoked via Stash: {self.via_stash}")
 
         print(
             f"{padding} Result: {self.result_icon()} {self.state} {self.result_icon()}"
         )
 
+    def log_query_result(self):
+        if not self.issuer:
+            log.warning(f"{self.name} Unknown issuer")
+            return
+
+        logdata = {
+            "name": self.name,
+            "issuer": self.issuer["subject"],
+            "state": self.state,
+            "state_icon": self.result_icon(),
+            "enrolled_in_crlite": "✅" if self.issuer["crlite_enrolled"] else "❌",
+            "cert_id": self.cert_id,
+            "via_filter": self.via_filter,
+            "via_stash": self.via_stash,
+        }
+
+        if self.state == "Revoked":
+            log.warning(logdata)
+        else:
+            log.info(logdata)
+
 
 class CRLiteQuery(object):
     def __init__(self, *, intermediates_db, crlite_db):
@@ -500,6 +543,16 @@ class CRLiteQuery(object):
                 "issuer"
             )
             issuer = self.intermediates_db.issuer_by_DN(issuerDN)
+            if not issuer:
+                yield CRLiteQueryResult(
+                    name=name,
+                    issuer=None,
+                    cert_id=None,
+                    crlite_db=self.crlite_db,
+                    not_before=None,
+                    not_after=None,
+                )
+                continue
 
             cert_id = CertId(issuer["issuerId"], serial_bytes)
 
